unit Mabdsave;

interface

uses
   SysUtils, WinTypes, WinProcs, Classes, Forms, DBTables, MabSave, IniFiles;

type
   TMabDBProperty = (dpFieldDisplayWidths, dpFieldIndexes, dpFieldVisibles, dpIndexFieldNames);
   TMabDBProperties = set of TMabDBProperty;
   TMabDBSave = class(TMabSave)
   private
      { Private declarations }
      FTable: TTable;
      FRemember: TMabDBProperties;
   protected
      { Protected declarations }
      procedure Notification(AComponent: TComponent; Operation: TOperation); override;
   public
      { Public declarations }
      constructor Create(AOwner: TComponent); override;

      procedure Load;
      procedure Save;
   published
      { Published declarations }
      property Table: TTable read FTable write FTable;
      property Filename;
      property Remember: TMabDBProperties read FRemember write FRemember;
   end;

implementation

constructor TMabDBSave.Create(AOwner: TComponent);
begin
   inherited Create(AOwner);

   FRemember := [dpFieldDisplayWidths, dpFieldIndexes, dpFieldVisibles, dpIndexFieldNames]
end;

procedure TMabDBSave.Notification(AComponent: TComponent; Operation: TOperation);
begin
   if (Operation = opRemove) and (AComponent = Table) then
      Table := nil;
end;

procedure TMabDBSave.Load;
var
   AList: TStringList;
   PName, PMesg: PChar;
   i: Integer;
   ASection: string;
begin
   try
      if IniFile = nil then
         raise Exception.Create('Filename must be have a value');
      if Table = nil then
         raise Exception.Create('Table must be specified');

      with Table do
      begin
         ASection := Owner.ClassName + '.' + Name + '.Fields';

         {
            We must use AList here, rather than TTable's
            FieldCount and Fields[] properties, since the
            index of each field may change
         }
         AList := TStringList.Create;
         try
            GetFieldNames(AList);
            for i := 0 to AList.Count - 1 do
               with FindField(AList.Strings[i]) do
               begin
                  if dpFieldDisplayWidths in Remember then
                     DisplayWidth := IniFile.ReadInteger(ASection,
                        FieldName + '.DisplayWidth', DisplayWidth);
                  if dpFieldVisibles in Remember then
                     Visible := IniFile.ReadBool(ASection,
                        FieldName + '.Visible', Visible);
                  if dpFieldIndexes in Remember then
                     Index := IniFile.ReadInteger(ASection,
                        FieldName + '.Index', Index);
               end;
         finally
            AList.Free;
         end;

         if dpIndexFieldNames in Remember then
         begin
            ASection := Owner.ClassName + '.' + Name + '.IndexFieldNames';
            IndexFieldNames := IniFile.ReadString(ASection, 'Name', '');
         end;
      end;
   except
      on E: Exception do
      begin
         PName := StrAlloc(Length(Name) + 1);
         PMesg := StrAlloc(Length(E.Message) + 1);
         StrPCopy(PName, Name);
         StrPCopy(PMesg, E.Message);
         Application.MessageBox(PMesg, PName, MB_ICONEXCLAMATION);
         StrDispose(PName);
         StrDispose(PMesg);
      end;
   end;
end;

procedure TMabDBSave.Save;
var
   PName, PMesg: PChar;
   i: Integer;
   ASection: string;
begin
   try
      if IniFile = nil then
         raise Exception.Create('Filename must be have a value');
      if Table = nil then
         raise Exception.Create('Table must be specified');

      with Table do
      begin
         ASection := Owner.ClassName + '.' + Name + '.Fields';
         for i := 0 to FieldCount - 1 do
            with Fields[i] do
            begin
               if dpFieldDisplayWidths in Remember then
                  IniFile.WriteInteger(ASection,
                     FieldName + '.DisplayWidth', DisplayWidth);
               if dpFieldIndexes in Remember then
                  IniFile.WriteInteger(ASection,
                     FieldName + '.Index', Index);
               if dpFieldVisibles in Remember then
                  IniFile.WriteBool(ASection,
                     FieldName + '.Visible', Visible);
            end;

         if dpIndexFieldNames in Remember then
         begin
            ASection := Owner.ClassName + '.' + Name + '.IndexFieldNames';
            IniFile.WriteString(ASection, 'Name', IndexFieldNames);
         end;
      end;
   except
      on E: Exception do
      begin
         PName := StrAlloc(Length(Name) + 1);
         PMesg := StrAlloc(Length(E.Message) + 1);
         StrPCopy(PName, Name);
         StrPCopy(PMesg, E.Message);
         Application.MessageBox(PMesg, PName, MB_ICONEXCLAMATION);
         StrDispose(PName);
         StrDispose(PMesg);
      end;
   end;
end;

end.
