unit MabPWDlg;

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, Buttons, DB, DBTables, ExtCtrls, DBCtrls, MabObj;

type
   TdlgMabPassword = class(TForm)
      btnCancel: TBitBtn;
      btnOk: TBitBtn;
      srcUser: TDataSource;
    grpMain: TGroupBox;
    lblUsername: TLabel;
    txtUser: TDBText;
    lblPassword: TLabel;
    edtPassword: TEdit;
    imgPassword: TImage;
      procedure btnOkClick(Sender: TObject);
   private
      { Private declarations }
      UserField: string;
      PasswordField: string;

      procedure SetButtonStyle(AButtonStyle: TButtonStyle);
      procedure SetPasswordEdit(APasswordEdit: TMabPasswordEdit);
  public
    { Public declarations }
    constructor Create(AOwner: TComponent;
      AUserTable: TTable; AUserField, APasswordField: string);

    function Execute: Boolean;

    property ButtonStyle: TButtonStyle write SetButtonStyle;
    property PasswordEdit: TMabPasswordEdit write SetPasswordEdit;
  end;

   TMabDBPasswordDlg = class(TComponent)
   private
      { Private declarations }
      FButtonStyle: TButtonStyle;
      FCaption: string;
      FFont: TFont;
      FParent: TForm;
      FParentFont: Boolean;
      FPasswordEdit: TMabPasswordEdit;
      FPasswordField: string;
      FUserField: string;
      FUserTable: TTable;

      procedure SetFont(AFont: TFont);
      procedure FontChange(Sender: TObject);
      procedure SetParentFont(AParentFont: Boolean);
      procedure SetPasswordField(APasswordField: string);
      procedure SetUserField(AUserField: string);
   protected
      { Protected declarations }
   public
      { Public declarations }
      constructor Create(AOwner: TComponent); override;
      destructor Destroy; override;
      procedure Notification(AComponent: TComponent; Operation: TOperation); override;

      function Execute: Boolean;

      property Parent: TForm read FParent;
   published
      { Published declarations }
      property ButtonStyle: TButtonStyle read FButtonStyle write FButtonStyle
         default bsAutoDetect;
      property Caption: string read FCaption write FCaption;
      property Font: TFont read FFont write SetFont;
      property ParentFont: Boolean read FParentFont write SetParentFont
         default True;
      property PasswordEdit: TMabPasswordEdit read FPasswordEdit
         write FPasswordEdit;
      property PasswordField: string read FPasswordField write SetPasswordField;
      property UserField: string read FUserField write FUserField;
      property UserTable: TTable read FUserTable write FUserTable;
   end;

implementation

{$R *.DFM}

constructor TdlgMabPassword.Create(AOwner: TComponent;
      AUserTable: TTable; AUserField, APasswordField: string);
begin
   inherited Create(AOwner);

   UserField := AUserField;
   PasswordField := APasswordField;

   txtUser.DataField := UserField;
   srcUser.DataSet := AUserTable;

   edtPassword.Height := 0;
end;

procedure TdlgMabPassword.SetButtonStyle(AButtonStyle: TButtonStyle);
begin
   btnOk.Style := AButtonStyle;
   btnCancel.Style := AButtonStyle;
end;

procedure TdlgMabPassword.SetPasswordEdit(APasswordEdit: TMabPasswordEdit);
begin
   with APasswordEdit do
   begin
      lblPassword.Caption := Prompt;
      edtPassword.CharCase := CharCase;
      edtPassword.MaxLength := MaxLength;
      edtPassword.PasswordChar := PasswordChar;
      edtPassword.Text := Text;
   end;
end;

function TdlgMabPassword.Execute: Boolean;
begin
   Result := (ShowModal = mrOk);
end;

procedure TdlgMabPassword.btnOkClick(Sender: TObject);
begin
   if srcUser.DataSet.FieldByName(PasswordField).AsString <> edtPassword.Text then
   begin
      Application.MessageBox('Incorrect password!', 'Access Denied', MB_ICONEXCLAMATION);
      ActiveControl := edtPassword;
      edtPassword.SelectAll;
      ModalResult := mrNone;
   end
   else
      ModalResult := btnOk.ModalResult;
end;

constructor TMabDBPasswordDlg.Create(AOwner: TComponent);
begin
   inherited Create(AOwner);

   if Owner is TForm then
      FParent := TForm(Owner)
   else
      raise Exception.Create('Owner must be of type TForm');

   FButtonStyle := bsAutoDetect;
   FFont := TFont.Create;
   FFont.Assign(Parent.Font);
   FFont.OnChange := FontChange;
   FParentFont := True;

   FPasswordEdit := TMabPasswordEdit.Create;
   FPasswordEdit.Prompt := '&Password:';

   FPasswordField := 'Password';
   FUserField := 'User';
end;

destructor TMabDBPasswordDlg.Destroy;
begin
   FFont.Free;
   FPasswordEdit.Free;

   inherited Destroy;
end;

procedure TMabDBPasswordDlg.Notification(AComponent: TComponent; Operation: TOperation);
begin
   if (Operation = opRemove) and (AComponent = UserTable) then
      UserTable := nil;
end;

procedure TMabDBPasswordDlg.SetFont(AFont: TFont);
begin
   if AFont <> FFont then
      FFont.Assign(AFont);
end;

procedure TMabDBPasswordDlg.FontChange(Sender: TObject);
begin
   ParentFont := False;
end;

procedure TMabDBPasswordDlg.SetParentFont(AParentFont: Boolean);
begin
   if AParentFont <> ParentFont then
   begin
      if AParentFont then
         Font := Parent.Font;
      FParentFont := AParentFont;
   end;
end;

procedure TMabDBPasswordDlg.SetPasswordField(APasswordField: string);
begin
   if (UserTable <> nil) and UserTable.Active
   and (UserTable.FindField(APasswordField) = nil) then
      raise EDatabaseError.Create('Field does not exist in UserTable')
   else
      FPasswordField := APasswordField
end;

procedure TMabDBPasswordDlg.SetUserField(AUserField: string);
begin
   if (UserTable <> nil) and UserTable.Active
   and (UserTable.FindField(AUserField) = nil) then
      raise EDatabaseError.Create('Field does not exist in UserTable')
   else
      FUserField := AUserField;
end;

function TMabDBPasswordDlg.Execute: Boolean;
var
   PasswordDlg: TdlgMabPassword;
begin
   if UserTable = nil then
      raise EDatabaseError.Create('UserTable must be set')
   else
   begin
      Result := False;
      PasswordDlg := TdlgMabPassword.Create(Self, UserTable, UserField, PasswordField);
      with PasswordDlg do
         try
            ButtonStyle  := Self.ButtonStyle;
            Caption      := Self.Caption;
            PasswordEdit := Self.PasswordEdit;
            Font.Name := ' ';
            Font.Assign(Self.Font);
            Result := Execute;
         finally
            Free;
         end;
   end;
end;

end.
